from django.db import models
from django.utils.translation import ugettext_lazy as _


class ImmortalQuerySet(models.query.QuerySet):
    """
    Represents a lazy database lookup for a set of objects.
    It updates "deleted" attribute instead deleting items.
    """
    def delete(self):
        self.update(deleted=True)


class ImmortalManager(models.Manager):
    def get_queryset(self):
        """
        Returns a new QuerySet object.  Subclasses can override this method
        to easily customize the behavior of the Manager.
        It filters by "deleted" attribute.
        """
        return ImmortalQuerySet(self.model, using=self._db).filter(deleted=False)

    get_query_set = get_queryset


class ImmortalModel(models.Model):
    """
    Implementation of undeletable model
    """
    deleted = models.BooleanField(_('deleted'), default=False)

    objects = ImmortalManager()
    baseobjects = models.Manager()

    def delete(self, using=None, **kwargs):
        self.deleted = True
        self.save()

    class Meta:
        abstract = True


from django.contrib import admin


class ImmortalAdmin(admin.ModelAdmin):
    exclude = ('deleted',)


class ImmortalTabularInline(admin.TabularInline):
    exclude = ('deleted',)


class ImmortalTabularInline(admin.StackedInline):
    exclude = ('deleted',)



from django.test import TestCase
from immortalmodel.models import ImmortalModel


class ImmortalTestCase(TestCase):
    """
    add immortalmodel to INSTALLED_APPS to run the following tests !!!
    """

    class Blashyrkh(ImmortalModel):
        pass

    def test_immortality(self):
        b = self.Blashyrkh()
        b.delete()
        self.assertRaises(self.Blashyrkh.DoesNotExist, self.Blashyrkh.objects.get, pk=b.pk)
        self.assertIsNotNone(self.Blashyrkh.baseobjects.get(pk=b.pk))

    def test_resuscitate(self):
        b = self.Blashyrkh()
        b.delete()
        self.assertRaises(self.Blashyrkh.DoesNotExist, self.Blashyrkh.objects.get, pk=b.pk)
        b.deleted = False
        b.save()
        self.assertIsNotNone(self.Blashyrkh.objects.get(pk=b.pk))

    def test_manually_deleted(self):
        b = self.Blashyrkh()
        b.deleted = True
        b.save()
        self.assertRaises(self.Blashyrkh.DoesNotExist, self.Blashyrkh.objects.get, pk=b.pk)



from setuptools import setup
import immortalmodel
setup(
    name='django-immortalmodel',
    version=immortalmodel.__version__,
    author='20Tab S.r.l.',
    author_email='info@20tab.com',
    description='A django model, manager and queryset implementing undeletable models',
    url='https://github.com/20tab/django-immortalmodel',
    packages=['immortalmodel'],
    license='MIT License',
    install_requires=[
        'Django >=2.1',
    ],
)
